#!/usr/bin/env python

"A dictionary example application."

import WebStack.Generic
from XSLForms.Resources.WebResources import \
    XSLFormsResource, output, resources, prepare_resources as xslforms_prepare_resources
import os

# Site map imports.

from WebStack.Resources.ResourceMap import MapResource
from WebStack.Resources.Selectors import EncodingSelector
from WebStack.Resources.Static import DirectoryResource

# Configuration setting.

encoding = "utf-8"

# Resource classes.

class DictionaryResource(XSLFormsResource):

    "A simple resource providing dictionary lookup."

    resource_dir = resources(__file__)
    template_resources = {
        "words" : output("words_template.xhtml")
        }
    in_page_resources = {
        "matches" : ("words", "words_output_entry.xsl", "matches-node"),
        #"word" : ("words", "words_output_word.xsl", "word-node")
        }

    def __init__(self, dict):

        "Initialise the resource with the given 'dict'."

        self.dict = dict

    def select_activity(self, trans, form):
        form.set_activity("words")

    def respond_to_input(self, trans, form):

        # Add and remove elements according to the selectors found.

        selectors = form.get_selectors()
        self.remove_elements(selectors.get("remove"))
        self.add_elements(selectors.get("add"), "entry")

    def init_document(self, trans, form):

        # Ensure all entries have a matches element.
        # Ensure all matches elements have at least one choice.
        # Copy selected matches to their corresponding text field.

        words = form.get_document()

        all_entries = words.xpath("words/entry")

        for entry in all_entries:
            matches_list = entry.xpath("matches")
            if len(matches_list) == 0:
                matches = words.createElement("matches")
                entry.appendChild(matches)
            else:
                matches = matches_list[0]

            if len(entry.xpath("matches/match-enum")) == 0:
                match_enum = words.createElement("match-enum")
                match_enum.setAttribute("word", "")
                matches.appendChild(match_enum)

        # Find requested search locations.

        selectors = form.get_selectors()
        in_page_resource = self.get_in_page_resource(trans)

        if selectors.has_key("search"):
            entries = selectors["search"]
        elif in_page_resource == "matches":
            entries = all_entries
        else:
            entries = []

        # Transform, adding dictionary information.

        for entry in entries:
            word = entry.getAttribute("word")
            if word != "":
                matches = entry.xpath("matches")[0]
                for found_word in self.dict.find(word):
                    match_enum = words.createElement("match-enum")
                    match_enum.setAttribute("word", found_word)
                    matches.appendChild(match_enum)

        # Copy selected values into text fields.
        # NOTE: Since libxml2dom does not guarantee node equality for two nodes
        # NOTE: referring to the same thing, we cannot just loop over all the
        # NOTE: entries and query whether they reside in the search locations.

        for entry in all_entries:
            matches = entry.xpath("matches")[0]
            if matches.hasAttribute("word"):
                word = matches.getAttribute("word")
                if word != "" and word.startswith(entry.getAttribute("word")):
                    entry.setAttribute("word", word)

# Site map initialisation.

def get_site(dict):

    """
    Return a simple Web site resource using the given 'dict' - a dictionary
    employed by the application.
    """

    # Get the main resource and the directory used by the application.

    dictionary_resource = DictionaryResource(dict)
    directory = dictionary_resource.resource_dir

    # Make a simple Web site.

    resource = MapResource({
        # Static resources:
        "scripts" : DirectoryResource(os.path.join(directory, "scripts"), {"js" : "text/javascript"}),
        # Main page and in-page resources:
        "" : dictionary_resource,
        "matches" : dictionary_resource
        })

    return EncodingSelector(resource, encoding)

# Resource preparation ahead of time - useful for making installations.

def prepare_resources():
    for cls in [DictionaryResource]:
        xslforms_prepare_resources(cls)

# vim: tabstop=4 expandtab shiftwidth=4
